@@ -2,7 +2,7 @@ import hildon
 import gtk
 import pge_window
 import cPickle
-import pge_window 
+import pge_window
 import os
 
 class Prefs():
@@ -14,32 +14,34 @@ class Prefs():
       f = open(os.path.expanduser("~")+"/.pygtkeditor",'r')
       self.prefs_dict = cPickle.load(f)
       if not self.prefs_dict.has_key('auto_rotate'):
-        self.prefs_dict['auto_rotate']=True                 
+        self.prefs_dict['auto_rotate']=True
       if not self.prefs_dict.has_key('show_lines'):
         self.prefs_dict['show_lines']=False
       if not self.prefs_dict.has_key('indent'):
         self.prefs_dict['indent']='  '
       if not self.prefs_dict.has_key('auto_clean_line_end'):
         self.prefs_dict['auto_clean_line_end']=False
-                  
+      if not self.prefs_dict.has_key('theme'):
+        self.prefs_dict['theme']='default'
+
     except:
       self.default()
-      
+
   def default(self):
     self.prefs_dict['hildon_text_completion']=True
     self.prefs_dict['default_language']='python'
-    self.prefs_dict['font_name']='Monospace'     
-    self.prefs_dict['font_size']='12'     
-    self.prefs_dict['auto_rotate']=True     
+    self.prefs_dict['font_name']='Monospace'
+    self.prefs_dict['font_size']='12'
+    self.prefs_dict['auto_rotate']=True
     self.prefs_dict['show_lines']=False
     self.prefs_dict['indent']='  '
     self.prefs_dict['auto_clean_line_end']=False
-    
+
   def store(self):
     f = open(os.path.expanduser("~")+"/.pygtkeditor",'w')
     prefs = cPickle.dump(self.prefs_dict,f)
 
-  def edit(self,parent_window):      
+  def edit(self,parent_window):
     dialog = gtk.Dialog('PyGTKEditor - Settings',parent_window,gtk.DIALOG_DESTROY_WITH_PARENT,(gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
 
     #hildon_text_completion
@@ -68,7 +70,7 @@ class Prefs():
 
     #default_language
     w_default_language = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
-                                       hildon.BUTTON_ARRANGEMENT_VERTICAL) 
+                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
     w_default_language.set_title("Default Language")
     w_default_language_selector = hildon.TouchSelectorEntry(text=True)
     languages = ['None']
@@ -80,10 +82,10 @@ class Prefs():
     w_default_language.set_selector(w_default_language_selector)
     if self.prefs_dict.has_key('default_language'):
       w_default_language.set_active(languages.index(self.prefs_dict['default_language']))
-    
+
     #Font Button
     w_font = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
-                                       hildon.BUTTON_ARRANGEMENT_VERTICAL) 
+                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
     w_font.set_title("Font")
     w_font_selector = hildon.TouchSelectorEntry(text=True)
     c = parent_window.create_pango_context()
@@ -97,10 +99,10 @@ class Prefs():
     if self.prefs_dict.has_key('font_name'):
       w_font.set_active(font_names.index(self.prefs_dict['font_name']))
       w_font.set_value(self.prefs_dict['font_name'])
-      
+
     #Font Size Button
     w_font_size = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
-                                       hildon.BUTTON_ARRANGEMENT_VERTICAL) 
+                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
     w_font_size.set_title("Size")
     w_font_size_selector = hildon.TouchSelectorEntry(text=True)
     font_sizes = []
@@ -115,7 +117,7 @@ class Prefs():
 
     #Indent Button
     w_indent = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
-                                       hildon.BUTTON_ARRANGEMENT_VERTICAL) 
+                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
     w_indent.set_title("Indent Style")
     w_indent_selector = hildon.TouchSelectorEntry(text=True)
     indent_style = ['2 spaces','4 spaces','Tabulation']
@@ -126,11 +128,27 @@ class Prefs():
     if self.prefs_dict.has_key('indent'):
       print self.prefs_dict['indent']
       w_indent.set_active(indent_value.index(self.prefs_dict['indent']))
-      w_indent.set_value(indent_style[indent_value.index(self.prefs_dict['indent'])])    
+      w_indent.set_value(indent_style[indent_value.index(self.prefs_dict['indent'])])
     else:
       w_indent.set_active(0)
       w_indent.set_value(indent_style[0])
 
+    #Theme Button
+    w_theme = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,
+                                       hildon.BUTTON_ARRANGEMENT_VERTICAL)
+    w_theme.set_title("Syntax Hilight Theme")
+    w_theme_selector = hildon.TouchSelectorEntry(text=True)
+    theme_list = ['default','dark',]
+    for i in theme_list :
+      w_theme_selector.append_text(i)
+    w_theme.set_selector(w_theme_selector)
+    if self.prefs_dict.has_key('theme'):
+      w_theme.set_active(theme_list.index(self.prefs_dict['theme']))
+      w_theme.set_value(self.prefs_dict['theme'])
+    else:
+      w_theme.set_active(0)
+      w_theme.set_value(theme_list[0])
+
     p2 = hildon.PannableArea()
     p = gtk.VBox()
 #    dialog.vbox.add(w_hildon_text_completion)
@@ -153,6 +171,7 @@ class Prefs():
     p.add(hbox)
     p.add(w_indent)
     p.add(w_auto_clean_line_end)
+    p.add(w_theme)
     p2.set_size_request(-1,300)
     p2.add_with_viewport(p)
     dialog.vbox.add(p2)
@@ -171,10 +190,11 @@ class Prefs():
       self.prefs_dict['font_size']= w_font_size_selector.get_current_text()
       self.prefs_dict['indent']= indent_value[w_indent_selector.get_active(0)]
       self.prefs_dict['auto_clean_line_end']=w_auto_clean_line_end.get_active()
+      self.prefs_dict['theme']= w_theme_selector.get_current_text()
       self.store()
       parent_window._parent.apply_prefs()
     dialog.destroy()
-        
+
 if __name__ == "__main__":
   prefs = Prefs()
   prefs.load()
