@@ -9,7 +9,7 @@
 # modify it under the terms of the GNU General Public License
 # as published by the Free Software Foundation; either version 2
 # of the License, or (at your option) any later version.
-#        
+#
 # Khertan (Benoit HERVIER) khertan@khertan.net
 
 import hildon
@@ -67,22 +67,24 @@ LANGUAGES = (('.R','R'),
             )
 
 class Window(hildon.Window):
-  def __init__(self,filepath=None,caller=None):
+  def __init__(self,filepath=None,caller=None,prefs=None,theme=None):
     hildon.Window.__init__ (self)
-    self.prefs = pge_preferences.Prefs()
-    self.prefs.load()
+    self.prefs = prefs
+    self.theme = theme
 
     if self.prefs.prefs_dict['auto_rotate']==True:
       self.rotation = FremantleRotation('net.khertan.pygtkeditor',self,mode=FremantleRotation.AUTOMATIC)
     else:
       self.rotation = FremantleRotation('net.khertan.pygtkeditor',self,mode=FremantleRotation.NEVER)
-    
+
     self.is_fullscreen = False
-    
+
+#    self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(theme['widget']['background']) )
+
     self._parent = caller
     self.filepath=filepath
 
-    self.code_editor=pge_editor.Editor(self._detect_language(self.filepath),font_name=self.prefs.prefs_dict['font_name'],font_size=self.prefs.prefs_dict['font_size'])
+    self.code_editor=pge_editor.Editor(self._detect_language(self.filepath),font_name=self.prefs.prefs_dict['font_name'],font_size=self.prefs.prefs_dict['font_size'],prefs=self.prefs.prefs_dict,theme=self.theme)
 
     self.toolbar = self.create_toolbar()
     self.add_toolbar(self.toolbar)
@@ -90,12 +92,16 @@ class Window(hildon.Window):
     menu = self.create_menu()
     menu.show_all()
     self.set_app_menu(menu)
-    
+
     self.create_findtoolbar()
 
     p1 = hildon.PannableArea()
     p1.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
 
+#    self.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse(theme['widget']['background']) )
+#    self.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(theme['widget']['background']) )
+#    self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(theme['widget']['background']) )
+
     p1.add(self.code_editor)
 
     #self.code_editor
@@ -103,12 +109,13 @@ class Window(hildon.Window):
     self.show_all()
     self.saved=True
 
-    print filepath
+#    print filepath
     if filepath!=None:
       self.set_title(os.path.basename(filepath))
       self.open_file(filepath)
     else:
-      self.set_title('Untitled')
+      self.set_title('*Untitled')
+      self.saved=False
 
     self.connect("key-press-event", self.on_key_press)
     self.code_editor.buffer.connect ('changed', self.on_modified)
@@ -131,14 +138,19 @@ class Window(hildon.Window):
           try:
             for cchild in achild.get_children():
               if type(cchild)==gtk.ComboBoxEntry:
-                self.set_focus(cchild.get_children()[0])                 
+                self.set_focus(cchild.get_children()[0])
           except:
             pass
       except:
         pass
 
-  def apply_prefs(self):
-    self.prefs.load()
+  def apply_prefs(self,prefs,theme):
+    self.prefs = prefs
+    self.theme = theme
+
+    #widget
+#    self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(theme['widget']['background']) )
+
     #rotate
     if self.prefs.prefs_dict['auto_rotate']==True:
       self.rotation.set_mode(FremantleRotation.AUTOMATIC)
@@ -146,21 +158,27 @@ class Window(hildon.Window):
     else:
       self.rotation.set_mode(FremantleRotation.NEVER)
 #      FremantleRotation('net.khertan.pygtkeditor',self,mode=FremantleRotation.NEVER)
-    #font / size
-    print self.prefs.prefs_dict['font_name']+" "+str(self.prefs.prefs_dict['font_size'])
-    self.code_editor.modify_font(pango.FontDescription (self.prefs.prefs_dict['font_name']+" "+str(self.prefs.prefs_dict['font_size'])))
-    
-    if self.prefs.prefs_dict['show_lines'] == True:
-      self.code_editor.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 30)
-      self.code_editor.line_view = self.code_editor.get_window(gtk.TEXT_WINDOW_LEFT)
-      if self.code_editor.expose_event_cb==None:
-        self.code_editor.expose_even_cb = self.code_editor.connect("expose_event", self.code_editor.line_numbers_expose)
-    else:
-      self.code_editor.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 0)
-#      self.code_editor.line_view = None
-      if self.code_editor.expose_event_cb!=None:
-        self.code_editor.disconnect(self.code_editor.expose_event_cb)
-  
+
+    #Apply prefs
+#    #font / size
+#    print self.prefs.prefs_dict['font_name']+" "+str(self.prefs.prefs_dict['font_size'])
+#    self.code_editor.modify_font(pango.FontDescription (self.prefs.prefs_dict['font_name']+" "+str(self.prefs.prefs_dict['font_size'])))
+#
+#    if self.prefs.prefs_dict['show_lines'] == True:
+#      self.code_editor.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 45)
+# #     self.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 50)
+#
+#      self.code_editor.line_view = self.code_editor.get_window(gtk.TEXT_WINDOW_LEFT)
+#      if self.code_editor.expose_event_cb==None:
+#        self.code_editor.expose_even_cb = self.code_editor.connect("expose_event", self.code_editor.line_numbers_expose)
+#    else:
+#      self.code_editor.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 0)
+##      self.code_editor.line_view = None
+#      if self.code_editor.expose_event_cb!=None:
+#        self.code_editor.disconnect(self.code_editor.expose_event_cb)
+    self.code_editor.apply_theme(self.theme)
+    self.code_editor.apply_prefs(prefs.prefs_dict)
+
   def onHideFind(self, widget):
     self.findToolBar.hide()
     self.searched_text = None
@@ -176,29 +194,40 @@ class Window(hildon.Window):
       return
     else:
       self.set_title('*'+title)
-       
-  def execute(self):
-    note = osso.SystemNote(self._parent.context)
-    result = note.system_note_infoprint("Launching "+ self.filepath +" ...")
 
-    fileHandle = open('/tmp/pygtkeditor.tmp', 'w')
-    fileHandle.write('#!/bin/sh\n')
-    fileHandle.write('cd '+os.path.dirname(self.filepath)+' \n')
-    fileHandle.write(self._detect_launch_language()+" \'"+self.filepath + "\'\n")
-    fileHandle.write('read -p "Press ENTER to continue ..." foo')
-    fileHandle.write('\nexit')
-    fileHandle.close()
-    commands.getoutput("chmod 777 /tmp/pygtkeditor.tmp")
-    Popen('/usr/bin/osso-xterm /tmp/pygtkeditor.tmp',shell=True,stdout=None)
+  def execute(self):
+    #ask for save if unsaved
+    if self.saved == False:
+      noteconf = hildon.hildon_note_new_confirmation(self,'Did you want to save this file before launching it ?')
+      if noteconf.run() == gtk.RESPONSE_OK:
+        self.save()
+      noteconf.destroy()
+
+    if self.filepath != None:
+      note = osso.SystemNote(self._parent.context)
+      result = note.system_note_infoprint("Launching "+ self.filepath +" ...")
+
+      fileHandle = open('/tmp/pygtkeditor.tmp', 'w')
+      fileHandle.write('#!/bin/sh\n')
+      fileHandle.write('cd '+os.path.dirname(self.filepath)+' \n')
+      fileHandle.write(self._detect_launch_language()+" \'"+self.filepath + "\'\n")
+      fileHandle.write('read -p "Press ENTER to continue ..." foo')
+      fileHandle.write('\nexit')
+      fileHandle.close()
+      commands.getoutput("chmod 777 /tmp/pygtkeditor.tmp")
+      Popen('/usr/bin/osso-xterm /tmp/pygtkeditor.tmp',shell=True,stdout=None)
+    else:
+      note = osso.SystemNote(self._parent.context)
+      result = note.system_note_infoprint("Unsaved file cannot be launched.")
 
   def _detect_launch_language(self):
     return 'python'
-    
+
   def _detect_language(self,filepath):
-        
+
     if filepath==None:
       return self.prefs.prefs_dict['default_language']
-                  
+
     for extension,lang in LANGUAGES:
       if filepath.endswith(extension.lower()):
         return lang
@@ -206,6 +235,9 @@ class Window(hildon.Window):
 
   def on_key_press(self, widget, event, *args):
     if (event.state==gtk.gdk.CONTROL_MASK):
+      #New : CTRL-N
+      if (event.keyval == gtk.keysyms.s):
+        self._parent.create_window()
       #Save : CTRL-S
       if (event.keyval == gtk.keysyms.s):
         self.save()
@@ -220,13 +252,13 @@ class Window(hildon.Window):
         self.open_dialog()
       #Searh : CTRL-F
       elif (event.keyval == gtk.keysyms.f):
-        self.onShowFind(widget) 
+        self.onShowFind(widget)
       #Duplicate line : CTRL-D
       elif (event.keyval == gtk.keysyms.d):
-        self.code_editor.duplicate_line() 
+        self.code_editor.duplicate_line()
       #Close : CTRL-W
       elif (event.keyval == gtk.keysyms.w):
-        self.destroy() 
+        self.destroy()
       #Show Info : CTRL-I
       elif (event.keyval == gtk.keysyms.i):
         info = hildon.FileDetailsDialog(self,self.filepath)
@@ -284,10 +316,10 @@ class Window(hildon.Window):
   def save_as(self):
     fsm = hildon.FileSystemModel()
 #    fc = hildon.FileChooserDialog(self, gtk.FILE_CHOOSER_ACTION_SAVE,fsm)
-    fc = gobject.new(hildon.FileChooserDialog, action=gtk.FILE_CHOOSER_ACTION_SAVE)      
+    fc = gobject.new(hildon.FileChooserDialog, action=gtk.FILE_CHOOSER_ACTION_SAVE)
     if self.filepath != None :
       fc.set_current_folder(os.path.dirname(self.filepath))
-    else: 
+    else:
       fc.set_current_folder(self._parent._last_opened_folder)
     fc.set_show_hidden(True)
     fc.set_do_overwrite_confirmation(False)
@@ -305,9 +337,14 @@ class Window(hildon.Window):
     if fc.run()==gtk.RESPONSE_OK:
       filepath = fc.get_filename()
       fc.destroy()
-      self.save_file(filepath)
-      manager = gtk.recent_manager_get_default()                         
-      manager.add_item('file://'+filepath)                               
+      if type(filepath) == str: # fix #19 - fix #23
+        self.save_file(filepath)
+        manager = gtk.recent_manager_get_default()
+        manager.add_item('file://'+filepath)
+      else:
+        note = osso.SystemNote(self._parent.context)
+        result = note.system_note_dialog('An error occurs saving file :\n'+str(filepath))
+
     else:
       fc.destroy()
 
@@ -330,7 +367,7 @@ class Window(hildon.Window):
       print e
       note = osso.SystemNote(self._parent.context)
       result = note.system_note_dialog('An error occurs opening file :\n'+str(e))
-  
+
   def create_toolbar(self):
     toolbar = gtk.Toolbar()
 #    label = 'Edit'
@@ -340,7 +377,7 @@ class Window(hildon.Window):
 #
 #    toolitem = gtk.ToggleToolButton()
 #    toolitem.set_icon_widget(i)
-#    toolitem.connect("toggled", self.toolbar_button_clicked, label)    
+#    toolitem.connect("toggled", self.toolbar_button_clicked, label)
 #    toolbar.insert(toolitem, 0)
 
     label = 'Comment'
@@ -357,11 +394,11 @@ class Window(hildon.Window):
     toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 1)
 
-    label = 'Unindent' 
+    label = 'Unindent'
     toolitem = gtk.ToolButton(gtk.image_new_from_stock(gtk.STOCK_UNINDENT,
                                  gtk.ICON_SIZE_LARGE_TOOLBAR),
                                  label)
-    toolitem.connect("clicked", self.toolbar_button_clicked, label)    
+    toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 2)
 
     toolbar.insert(gtk.SeparatorToolItem(),3)
@@ -370,7 +407,7 @@ class Window(hildon.Window):
     toolitem = gtk.ToolButton(gtk.image_new_from_stock(gtk.STOCK_SAVE,
                                  gtk.ICON_SIZE_LARGE_TOOLBAR),
                                  label)
-    toolitem.connect("clicked", self.toolbar_button_clicked, label)    
+    toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 4)
 
     toolbar.insert(gtk.SeparatorToolItem(),5)
@@ -379,14 +416,14 @@ class Window(hildon.Window):
     toolitem = gtk.ToolButton(gtk.image_new_from_stock(gtk.STOCK_FIND,
                                  gtk.ICON_SIZE_LARGE_TOOLBAR),
                                  label)
-    toolitem.connect("clicked", self.toolbar_button_clicked, label)    
+    toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 6)
 
     label = 'Fullscreen'
     toolitem = gtk.ToolButton(gtk.image_new_from_stock(gtk.STOCK_FULLSCREEN,
                                  gtk.ICON_SIZE_LARGE_TOOLBAR),
                                  label)
-    toolitem.connect("clicked", self.toolbar_button_clicked, label)    
+    toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 7)
 
     toolbar.insert(gtk.SeparatorToolItem(),8)
@@ -395,7 +432,7 @@ class Window(hildon.Window):
     toolitem = gtk.ToolButton(gtk.image_new_from_stock(gtk.STOCK_EXECUTE,
                                  gtk.ICON_SIZE_LARGE_TOOLBAR),
                                  label)
-    toolitem.connect("clicked", self.toolbar_button_clicked, label)    
+    toolitem.connect("clicked", self.toolbar_button_clicked, label)
     toolbar.insert(toolitem, 9)
 
     return toolbar
@@ -407,7 +444,7 @@ class Window(hildon.Window):
       self.save_as()
 
   def toolbar_button_clicked (self,toolbutton, label):
-    print "Toolbar button clicked : %s" % label
+#    print "Toolbar button clicked : %s" % label
     if label=='Save':
       self.save()
     elif label=='Indent':
@@ -417,7 +454,7 @@ class Window(hildon.Window):
     elif label=='Unindent':
       self.code_editor.unindent_tab()
     elif label=='Search':
-      self.onShowFind(toolbutton) 
+      self.onShowFind(toolbutton)
     elif label=='Fullscreen':
       self.is_fullscreen = not self.is_fullscreen
       if self.is_fullscreen==True:
@@ -426,19 +463,19 @@ class Window(hildon.Window):
         self.unfullscreen()
     elif label=='Execute':
       self.execute()
-      
+
   def menu_button_clicked(self,button, label):
     if label == 'New':
-      self._parent.create_window() 
+      self._parent.create_window()
     elif label=='Recent':
       filepath = pge_recentchooser.Dialog().get()
       if filepath!=None:
         self._parent.create_window(filepath)
 #       fc = gtk.RecentChooserDialog("Recent Documents", self, None,(gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
-#       if fc.run()==gtk.RESPONSE_ACCEPT:                                    
+#       if fc.run()==gtk.RESPONSE_ACCEPT:
 #         filepath = fc.get_current_item().get_uri()[7::]
-#         fc.destroy()                                                   
-#         self._parent.create_window(filepath)   
+#         fc.destroy()
+#         self._parent.create_window(filepath)
 #       else:
 #         fc.destroy()
     elif label=='About':
@@ -470,11 +507,11 @@ class Window(hildon.Window):
     self.open_menu_button.connect("clicked", self.menu_button_clicked, label)
     menu.append(self.open_menu_button)
     self.saveas_menu_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-    self.recent_menu_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)     
-    label = 'Recent'                                                     
-    self.recent_menu_button.set_label(label)                             
+    self.recent_menu_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+    label = 'Recent'
+    self.recent_menu_button.set_label(label)
     self.recent_menu_button.connect("clicked", self.menu_button_clicked, label)
-    menu.append(self.recent_menu_button)                                 
+    menu.append(self.recent_menu_button)
     label = 'Save as'
     self.saveas_menu_button.set_label(label)
     self.saveas_menu_button.connect("clicked", self.menu_button_clicked, label)
@@ -489,10 +526,10 @@ class Window(hildon.Window):
     self.settings_menu_button.set_label(label)
     self.settings_menu_button.connect("clicked", self.menu_button_clicked, label)
     menu.append(self.settings_menu_button)
-    self.help_menu_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO) 
-    label = 'Help'                                             
-    self.help_menu_button.set_label(label)                   
+    self.help_menu_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+    label = 'Help'
+    self.help_menu_button.set_label(label)
     self.help_menu_button.connect("clicked", self.menu_button_clicked, label)
-    menu.append(self.help_menu_button)                  
-                                              
+    menu.append(self.help_menu_button)
+
     return menu
