@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2009 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2010 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,11 @@ import dbus.glib
 import hildon
 import osso
 
+# Replace this with your own gettext() functionality
+#import gpodder
+#_ = gpodder.gettext
+
+
 class FremantleRotation(object):
     """thp's screen rotation for Maemo 5
 
@@ -50,6 +55,11 @@ class FremantleRotation(object):
     _MCE_REQUEST_PATH = '/com/nokia/mce/request'
     _MCE_REQUEST_IF = 'com.nokia.mce.request'
 
+    # sysfs device name for the keyboard slider switch
+    KBD_SLIDER = '/sys/devices/platform/gpio-switch/slide/state'
+    _KBD_OPEN = 'open'
+    _KBD_CLOSED = 'closed'
+
     def __init__(self, app_name, main_window=None, version='1.0', mode=0):
         """Create a new rotation manager
 
@@ -62,9 +72,9 @@ class FremantleRotation(object):
         self._main_window = main_window
         self._stack = hildon.WindowStack.get_default()
         self._mode = -1
-        #Khertan add s for initial position
-        self._last_dbus_orientation = self.get_current_orientation()
-        app_id = '-'.join((app_name, str(self.__class__.__name__)))
+        self._last_dbus_orientation = None
+        self._keyboard_state = self._get_keyboard_state()
+        app_id = '-'.join((app_name, self.__class__.__name__))
         self._osso_context = osso.Context(app_id, version, False)
         program = hildon.Program.get_instance()
         program.connect('notify::is-topmost', self._on_topmost_changed)
@@ -72,20 +82,13 @@ class FremantleRotation(object):
         system_bus.add_signal_receiver(self._on_orientation_signal, \
                 signal_name='sig_device_orientation_ind', \
                 dbus_interface='com.nokia.mce.signal', \
-                path='/com/nokia/mce/signal')        
+                path='/com/nokia/mce/signal')
+        system_bus.add_signal_receiver(self._on_keyboard_signal, \
+                signal_name='Condition', \
+                dbus_interface='org.freedesktop.Hal.Device', \
+                path='/org/freedesktop/Hal/devices/platform_slide')
         self.set_mode(mode)
 
-    #Khertan add s for initial position
-    def get_current_orientation(self):
-        f = open("/sys/class/i2c-adapter/i2c-3/3-001d/coord", 'r' )
-        x,y,z = [int(w) for w in f.readline().split()]
-        f.close()
-        
-        if abs(x)<500: 
-          return self._LANDSCAPE
-        else:
-          return self._PORTRAIT
-          
     def get_mode(self):
         """Get the currently-set rotation mode
 
@@ -108,14 +111,13 @@ class FremantleRotation(object):
                 self._last_dbus_orientation = self._orientation
                 # Tell MCE that we don't need the accelerometer anymore
                 self._send_mce_request(self._DISABLE_ACCEL)
-                
 
             if new_mode == self.NEVER:
                 self._orientation_changed(self._LANDSCAPE)
-            elif new_mode == self.ALWAYS:
+            elif new_mode == self.ALWAYS and \
+                    self._keyboard_state != self._KBD_OPEN:
                 self._orientation_changed(self._PORTRAIT)
             elif new_mode == self.AUTOMATIC:
-                
                 # Restore the last-known "automatic" orientation
                 self._orientation_changed(self._last_dbus_orientation)
                 # Tell MCE that we need the accelerometer again
@@ -169,12 +171,33 @@ class FremantleRotation(object):
 
         self._orientation = orientation
 
+    def _get_keyboard_state(self):
+        return open(self.KBD_SLIDER).read().strip()
+
+    def _keyboard_state_changed(self):
+        state = self._get_keyboard_state()
+
+        if state == self._KBD_OPEN:
+            self._orientation_changed(self._LANDSCAPE)
+        elif state == self._KBD_CLOSED:
+            if self._mode == self.AUTOMATIC:
+                self._orientation_changed(self._last_dbus_orientation)
+            elif self._mode == self.ALWAYS:
+                self._orientation_changed(self._PORTRAIT)
+
+        self._keyboard_state = state
+
+    def _on_keyboard_signal(self, condition, button_name):
+        if condition == 'ButtonPressed' and button_name == 'cover':
+            self._keyboard_state_changed()
+
     def _on_orientation_signal(self, orientation, stand, face, x, y, z):
         if orientation in (self._PORTRAIT, self._LANDSCAPE):
-            if self._mode == self.AUTOMATIC:
+            if self._mode == self.AUTOMATIC and \
+                    self._keyboard_state != self._KBD_OPEN:
                 # Automatically set the rotation based on hardware orientation
                 self._orientation_changed(orientation)
-            else:
-                # Ignore orientation changes for non-automatic modes, but save
-                # the current orientation for "automatic" mode later on
-                self._last_dbus_orientation = orientation
\ No newline at end of file
+
+            # Save the current orientation for "automatic" mode later on
+            self._last_dbus_orientation = orientation
+
