@@ -15,32 +15,42 @@
 import hildon
 import gtk
 import osso
-import pango
+#import pango
 import os
+import pge_theme
+import pge_preferences
 
 gtk.gdk.threads_init()
 
-from portrait import FremantleRotation
+#from portrait import FremantleRotation
 import pge_window
 import sys
+import pge_report
 
 class PyGTKEditor:
 
-  VERSION = '3.0.14'
+  VERSION = '3.0.19'
+  pge_report.install_excepthook("PyGTKEditor", VERSION)
 
   def __init__(self):
 
     self.app = hildon.Program()
-    self.context = osso.Context('net.khertan.pygtkeditor',self.VERSION, False)
+    self.context = osso.Context('net.khertan.pygtkeditor', self.VERSION, False)
     self.app.set_can_hibernate(True)
     self.window_list = None
     self._manage_theme()
+    self.theme = None
+
+    self.prefs = pge_preferences.Prefs()
+    self.prefs.load()
+
+    self.load_theme()
 
     self._last_opened_folder = '/home/user/MyDocs/'
 
     if self.context:
       self.osso_rpc = osso.Rpc(self.context)
-      self.osso_rpc.set_rpc_callback("net.khertan.pygtkeditor","/net/khertan/pygtkeditor","net.khertan.pygtkeditor", self.cb_mime_open, self.context)
+      self.osso_rpc.set_rpc_callback("net.khertan.pygtkeditor", "/net/khertan/pygtkeditor",  "net.khertan.pygtkeditor", self.cb_mime_open, self.context)
 
     argv = sys.argv
     argv.pop(0)
@@ -69,7 +79,7 @@ class PyGTKEditor:
 
   def onAbout(self, widget):
     import about
-    dial = about.Dialog('PyGTKEditor',self.VERSION,'A source code editor for Maemo','pygtkeditor','GPLv2','http://khertan.net/','http://bugs.maemo.org')
+    dial = about.Dialog('PyGTKEditor', self.VERSION,  'A source code editor for Maemo' , 'pygtkeditor', 'GPLv2','http://khertan.net/','http://bugs.maemo.org')
     dial.run()
     dial.destroy()
 
@@ -108,26 +118,37 @@ class PyGTKEditor:
       iconfactory.add(stock_id, iconset)
 
     iconset = gtk.IconSet(gtk.gdk.pixbuf_new_from_file('/usr/share/icons/hicolor/48x48/hildon/pygtkeditor-increase_indent.png'))
-    iconfactory.add(gtk.STOCK_INDENT,iconset)
+    iconfactory.add(gtk.STOCK_INDENT, iconset)
     iconfactory.add_default()
     iconset = gtk.IconSet(gtk.gdk.pixbuf_new_from_file('/usr/share/icons/hicolor/48x48/hildon/pygtkeditor-decrease_indent.png'))
-    iconfactory.add(gtk.STOCK_UNINDENT,iconset)
+    iconfactory.add(gtk.STOCK_UNINDENT, iconset)
     iconfactory.add_default()
+    return
 
-  def close_win(self,widget,*data):
+  def load_theme(self):
+    if self.prefs.prefs_dict.has_key('theme'):
+        theme_name = self.prefs.prefs_dict['theme']+'.theme'
+    else:
+        print 'DEFAULT THEME loaded'
+        theme_name = 'default.theme'
+    self.theme = pge_theme.Theme(theme_name).theme_dict
+    return
+
+  def close_win(self, widget,  *data):
     if widget.saved == False:
       noteconf = hildon.hildon_note_new_confirmation(hildon.Window(),'Did you want to save this unsaved file ?')
       if noteconf.run() == gtk.RESPONSE_OK:
         widget.save()
       noteconf.destroy()
+    return
 
-  def destroy_win(self,widget,*data):
+  def destroy_win(self, widget,  *data):
     self.window_list.remove(widget)
     if len(self.window_list)==0:
       gtk.main_quit()
 
-  def create_window(self,filepath=None):
-    newwin = pge_window.Window(filepath,self)
+  def create_window(self, filepath=None):
+    newwin = pge_window.Window(filepath, self,  prefs=self.prefs , theme=self.theme)
     newwin.connect("destroy", self.destroy_win)
     newwin.connect("delete-event", self.close_win)
     if self.window_list==None:
@@ -136,8 +157,11 @@ class PyGTKEditor:
     self.app.add_window(newwin)
 
   def apply_prefs(self):
+    self.prefs.load()
+    self.load_theme()
+
     for window in self.window_list:
-      window.apply_prefs()
+      window.apply_prefs(self.prefs, self.theme)
 
 if __name__ == "__main__":
   PyGTKEditor()
